# Nodes

There are several nodes than can be used to communicate with your devices:
* [Devices](#devices)
* [Device (generic)](#device)
* [Firmware](#firmware)
* [Power state (in)](#power-state-in)
* [Power state (out)](#power-state-out)
* [Channels](#channels)
* [Event listener](#event-listener)
* [Power usage](#power-usage)
* [Humidity](#humidity)
* [Temperature](#temperature)
* [Temperature and humidity](#temp-hum)

#### Devices node :id=devices

![](images/nodes/devices.png)

Using this node you can retrieve all the devices attached to your account.

Result example:
```json
[
    {
        "_id": "5df2cbda08eb57bea4d6c70f",
        "name": "Kitchen Lamp"
        "type": "10",
        "deviceid: "12345678",
        ...
    },
    ...
]
```

#### Device node (generic) :id=device

![](images/nodes/device.png)

You can communicate with your devices using RPC-like syntax.

A list of all methods you can find here: https://ewelink-api.now.sh/docs/available-methods 

The input payload should look like:
```json
{
    "method": "<method name>",
    "params": [<params>],
}
```

Simple example to turn on a **Sonoff Basic** relay:
```json
{
    "method": "setDevicePowerState",
    "params": ["on"]
}
```

#### Firmware node :id=firmware

![](images/nodes/firmware.png)

Returns the current firmware version of a device.

Result example:
```json
{
    "status": "ok",
    "fwVersion": "1.6.0"
}
```

#### Power state node (in) :id=power-state-in

![](images/nodes/power-state-in.png)

Reads the current power state for a channel of a device.

Result example:
```json
{
    "status": "ok",
    "state": "on"
}
```

#### Power state node (out) :id=power-state-out

![](images/nodes/power-state-out.png)

Sets the power state for a channel of a device. The payload can be any of the following strings:
* on
* off
* toggle

Result example:
```json
{
    "status": "ok",
    "state": "on"
}
```

#### Channels node :id=channels

![](images/nodes/channels.png)

Returns the number of the channels for the device.

Result example:
```json
{
    "status": "ok",
    "switchesAmount": 1
}
```

#### Event listener node :id=event-listener

![](images/nodes/event-listener.png)

Listens to events of a devices. When a change occures it will return the event.
By default it listens for events generated by all the devices, but you can specify a single device ID to listen on.

Result example:
```json
{
    "action": "update",
    "deviceid": "12345678",
    "params": {
        switch: "on"
    }
}
```

#### Power usage node :id=power-usage

![](images/nodes/power-usage.png)

Returns the current month power usage on the device.

#### [Humidity node](#humidity-node)

![](images/nodes/humidity.png)

Returns the humidity value from the sensor connected to the device.

Result example:
```json
{
    "status": "ok",
    "humidity": "40.15"
}
```

#### Temperature node :id=temperature

![](images/nodes/temperature.png)

Returns the temperature value from the sensor connected to the device.

Result example:
```json
{
    "status": "ok",
    "temperature": "21.79"
}
```

#### Temperature and humidity node :id=temp-hum

![](images/nodes/temp-hum.png)

Returns the temperature and humidity value from the sensor connected to the device.

Result example:
```json
{
    "status": "ok",
    "temperature": "21.79",
    "humidity": "40.15"
}
```

[← Go back to the configuration](configuration.md)
